using System;
using System.Collections.Generic;
using System.Text;
using ImageSharp.Images;

namespace ImageSharp
{
    class Arithmetics
    {
        /// <summary>
        /// Subtraktion des zweiten bergebenen Grauwertbildes vom ersten Bild.
        /// </summary>
        /// <param name="img1">Grauwertbild, von dem substrahiert werden soll</param>
        /// <param name="img2">Grauwertbild, das substrahiert werden soll</param>
        public static ImageGray Sub(ImageGray img1, ImageGray img2)
        {
            return Sub(img1, img2, 1.0, 1.0);
        }

        /// <summary>
        /// Subtraktion des zweiten bergebenen Grauwertbildes vom ersten Bild mit den angegebenen Faktoren.
        /// </summary>
        /// <param name="img1">Grauwertbild, von dem substrahiert werden soll</param>
        /// <param name="img2">Grauwertbild, das substrahiert werden soll</param>
        /// <param name="fac1">Vorfaktor des ersten Grauwertbildes</param>
        /// <param name="fac2">Vorfaktor des zu subtrahierenden zweiten Bildes</param>
        public static ImageGray Sub(ImageGray img1, ImageGray img2, double fac1, double fac2)
        {
            if (img1.Width != img2.Width || img1.Height != img2.Height)
            {
                throw new Exception("Images to subtract must have the same dimensions.");
            }

            ImageGray resultImg = new ImageGray(img1.Width, img1.Height);
            for (int y = 0; y < img1.Height; y++)
            {
                for (int x = 0; x < img1.Width; x++)
                {
                    // Subtraktion unter Einbeziehung der Vorfaktoren
                    resultImg.Data[y, x] = (byte)Math.Max(Math.Min(Math.Abs(fac1 * img1.Data[y, x] - fac2 * img2.Data[y, x]), 255), 0);
                }
            }

            return resultImg;
        }

        /// <summary>
        /// Addition der beiden bergebenen Grauwertbilder.
        /// </summary>
        /// <param name="img1">Grauwertbild 1</param>
        /// <param name="img2">Grauwertbild 2</param>
        public static ImageGray Add(ImageGray img1, ImageGray img2)
        {
            return Add(img1, img2, 1.0, 1.0);
        }

        /// <summary>
        /// Addition der beiden bergebenen Grauwertbilder mit den angegebenen Faktoren.
        /// </summary>
        /// <param name="img1">Grauwertbild 1</param>
        /// <param name="img2">Grauwertbild 2</param>
        /// <param name="fac1">Vorfaktor des ersten Grauwertbildes</param>
        /// <param name="fac2">Vorfaktor des zweiten Grauwertbildes</param>
        public static ImageGray Add(ImageGray img1, ImageGray img2, double fac1, double fac2)
        {
            if (img1.Width != img2.Width || img1.Height != img2.Height)
            {
                throw new Exception("Images to subtract must have the same dimensions.");
            }

            ImageGray resultImg = new ImageGray(img1.Width, img1.Height);
            for (int y = 0; y < img1.Height; y++)
            {
                for (int x = 0; x < img1.Width; x++)
                {
                    // Subtraktion unter Einbeziehung der Vorfaktoren
                    resultImg.Data[y, x] = (byte)Math.Max(Math.Min(Math.Abs(fac1 * img1.Data[y, x] + fac2 * img2.Data[y, x]), 255), 0);
                }
            }

            return resultImg;
        }
    }
}
