using System;
using System.Collections.Generic;
using System.Text;
using ImageSharp.Images;
using System.Drawing;
using ImageSharp.Segmentations.Binarizations;
using ImageSharp;
using ImageSharpApp.AlgProcessor;

namespace ImageSharpApp
{
    class ProcessController
    {
        private FormImage form = null;

        private GeoSeamCarvingProc geoSeamCarving = new GeoSeamCarvingProc();
        private GrayContrastAdaptProc grayContrastAdapt = new GrayContrastAdaptProc();
        private GrayContrastStretchProc grayContrastStretch = new GrayContrastStretchProc();
        private GrayGammaCorrectionProc grayGammaCorrection = new GrayGammaCorrectionProc();
        private GrayLUTApplicationProc grayLUTApplication = new GrayLUTApplicationProc();
        private GrayFunctionApplicationProc grayFunctionApplication = new GrayFunctionApplicationProc();
        private GrayImgInversionProc grayImgInversion = new GrayImgInversionProc();
        private GrayHistEqualizationProc grayHistEqualization = new GrayHistEqualizationProc();
        private GrayHistLocalEqualizationProc grayHistLocalEq = new GrayHistLocalEqualizationProc();
        private GrayHistSpecificationProc grayHistSpecification = new GrayHistSpecificationProc();
        private SegThresholdingProc segThresholding = new SegThresholdingProc();
        private SegOtsuThresholdingProc segOtsuThresholding = new SegOtsuThresholdingProc();
        private SegADBProc segADB = new SegADBProc();
        private FilterLinearGaussProc filterLinearGauss = new FilterLinearGaussProc();
        private FilterLinearAdaptiveGaussProc filterLinearAdapGauss = new FilterLinearAdaptiveGaussProc();
        private FilterLinearLowpassProc filterLinearLowpass = new FilterLinearLowpassProc();
        private FilterLinearHighpassProc filterLinearHighpass = new FilterLinearHighpassProc();
        private FilterLinearHighboostProc filterLinearHighboost = new FilterLinearHighboostProc();
        private FilterLinearLaplacianProc filterLinearLaplace = new FilterLinearLaplacianProc();
        private FilterLinearLaplacianSharpenProc filterLinearLaplaceSharpen = new FilterLinearLaplacianSharpenProc();
        private FilterLinearUserDefinedProc filterLinearUserDefined = new FilterLinearUserDefinedProc();
        private FilterNonLinearMedianProc filterNonLinearMedian = new FilterNonLinearMedianProc();
        private FilterNonLinearUnsharpenProc filterNonLinearUnsharpen = new FilterNonLinearUnsharpenProc();
        private FilterNonLinearISEFProc filterNonLinearISEF = new FilterNonLinearISEFProc();
        private FilterNonLinearHighboostProc filterNonLinearHighboost = new FilterNonLinearHighboostProc();
        private FilterNonLinearHighboost2Proc filterNonLinearHighboost2 = new FilterNonLinearHighboost2Proc();
        private FilterSpectralLaplaceProc filterSpectralLaplace = new FilterSpectralLaplaceProc();
        private FilterSpectralLoGProc filterSpectralLoG = new FilterSpectralLoGProc();
        private FilterSpectralBandpassProc filterSpectralBandpass = new FilterSpectralBandpassProc();
        private FilterSpectralLowpassProc filterSpectralLowpass = new FilterSpectralLowpassProc();
        private FilterSpectralHighpassProc filterSpectralHighpass = new FilterSpectralHighpassProc();
        private FilterSpectralSmoothBandpassProc filterSpectralSmoothBandpass = new FilterSpectralSmoothBandpassProc();
        private FilterSpectralSmoothLowpassProc filterSpectralSmoothLowpass = new FilterSpectralSmoothLowpassProc();
        private FilterSpectralSmoothHighpassProc filterSpectralSmoothHighpass = new FilterSpectralSmoothHighpassProc();
        private EdgesCannyProc edgesCanny = new EdgesCannyProc();
        private EdgesShenCastanProc edgesShenCastan = new EdgesShenCastanProc();
        private EdgesMarrHildrethProc edgesMarrHildreth = new EdgesMarrHildrethProc();
        private EdgesRobertsProc edgesRoberts = new EdgesRobertsProc();
        private EdgesPrewittProc edgesPrewitt = new EdgesPrewittProc();
        private EdgesSobelProc edgesSobel = new EdgesSobelProc();
        private EdgesLaplaceProc edgesLaplace = new EdgesLaplaceProc();
        
        public ProcessController(FormImage form)
        {
            this.form = form;
        }

        public bool Process(IPAlgorithms algo)
        {
            if (IPAlgorithmInfo.NeedsGrayImg(algo))
            {
                if (!checkImgGrayState())
                    return false;
            }
            else
            {
                if (!checkImgState())
                    return false;
            }

            switch (algo)
            {
                case IPAlgorithms.GeoSeamCarving:
                    this.geoSeamCarving.Apply(this.form.Img);
                    this.form.ShowImage();
                    this.form.FitFormToImage();
                    break;
                case IPAlgorithms.GrayContrastAdapt:
                    this.grayContrastAdapt.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayContrastStretch:
                    this.grayContrastStretch.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayGammaCorrection:
                    this.grayGammaCorrection.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayLUTApplication:
                    this.grayLUTApplication.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayFunctionApplication:
                    this.grayFunctionApplication.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayImgInversion:
                    this.grayImgInversion.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayHistEqualization:
                    this.grayHistEqualization.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayHistLocalEqualization:
                    this.grayHistLocalEq.Apply(this.form.Img);
                    break;
                case IPAlgorithms.GrayHistSpecification:
                    this.grayHistSpecification.Apply(this.form.Img);
                    break;
                case IPAlgorithms.SegThresholding:
                    this.segThresholding.Apply(this.form.Img);
                    break;
                case IPAlgorithms.SegOtsuThresholding:
                    this.segOtsuThresholding.Apply(this.form.Img);
                    break;
                case IPAlgorithms.SegADB:
                    this.segADB.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearGauss:
                    this.filterLinearGauss.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearAdaptiveGauss:
                    this.filterLinearAdapGauss.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearLowpass:
                    this.filterLinearLowpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearHighpass:
                    this.filterLinearHighpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearHighboost:
                    this.filterLinearHighboost.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearLaplacian:
                    this.filterLinearLaplace.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearLaplacianSharpen:
                    this.filterLinearLaplaceSharpen.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterLinearUserDefined:
                    this.filterLinearUserDefined.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterNonLinearMedian:
                    this.filterNonLinearMedian.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterNonLinearUnsharpen:
                    this.filterNonLinearUnsharpen.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterNonLinearISEF:
                    this.filterNonLinearISEF.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterNonLinearHighboost:
                    this.filterNonLinearHighboost.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterNonLinearHighboost2:
                    this.filterNonLinearHighboost2.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralLaplace:
                    this.filterSpectralLaplace.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralLoG:
                    this.filterSpectralLoG.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralBandpass:
                    this.filterSpectralBandpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralLowpass:
                    this.filterSpectralLowpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralHighpass:
                    this.filterSpectralHighpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralSmoothBandpass:
                    this.filterSpectralSmoothBandpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralSmoothLowpass:
                    this.filterSpectralSmoothLowpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.FilterSpectralSmoothHighpass:
                    this.filterSpectralSmoothHighpass.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesCanny:
                    this.edgesCanny.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesShenCastan:
                    this.edgesShenCastan.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesMarrHildreth:
                    this.edgesMarrHildreth.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesRoberts:
                    this.edgesRoberts.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesPrewitt:
                    this.edgesPrewitt.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesSobel:
                    this.edgesSobel.Apply(this.form.Img);
                    break;
                case IPAlgorithms.EdgesLaplace:
                    this.edgesLaplace.Apply(this.form.Img);
                    break;
                default:
                    throw new Exception("Algorithm not implemented or linked yet.");
            }

            this.form.ShowImage();
            return true;
        }

        private bool checkImgGrayState()
        {
            if (this.form != null)
            {
                if (!(this.form.Img is ImageGray))
                {
                    this.form.AddImgCloneToList();
                    this.form.MakeGrayImg();
                }
            }
            return checkImgState();
        }

        private bool checkImgGrayStateOnly()
        {
            if (this.form != null)
            {
                if (!(this.form.Img is ImageGray))
                {
                    this.form.AddImgCloneToList();
                    this.form.MakeGrayImg();
                }
            }
            return true;
        }

        private bool checkImgState()
        {
            if (this.form != null)
            {
                this.form.CutImgListOff();
                this.form.AddImgCloneToList();
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
